//$Id: UniqueKey.java,v 1.13 2005/01/29 03:03:19 oneovthafew Exp $
/**
 * Modification to ensure all constraints are initially deferred made 9/13/2007.
 * Original source from hibernate 2.1.8
 * Copyright 2007 Network Appliance, Inc., contribution by Philip Clay
 */
package net.sf.hibernate.mapping;

import java.util.Iterator;

import net.sf.hibernate.dialect.Dialect;
import net.sf.hibernate.engine.Mapping;
import net.sf.hibernate.util.StringHelper;
import net.sf.hibernate.dialect.Oracle9Dialect;

/**
 * A relational unique key constraint
 * @author Gavin King
 */
public class UniqueKey extends Constraint {
	
	public String sqlConstraintString(Dialect dialect) {
		StringBuffer buf = new StringBuffer("unique (");
		Iterator iter = getColumnIterator();
		while ( iter.hasNext() ) {
			buf.append( ( (Column) iter.next() ).getQuotedName(dialect) );
			if ( iter.hasNext() ) buf.append(StringHelper.COMMA_SPACE);
		}
		buf.append(StringHelper.CLOSE_PAREN);
		/*
		 * Modification by Network Appliance to ensure all constraints are initially deferred.
		 */
		if (dialect instanceof Oracle9Dialect) {
			buf.append(" initially deferred deferrable ");
		}
		return buf.toString();
	}
	
	public String sqlConstraintString(Dialect dialect, String constraintName, String defaultSchema) {
		StringBuffer buf = new StringBuffer(
			dialect.getAddPrimaryKeyConstraintString(constraintName)
		).append('(');
		Iterator iter = getColumnIterator();
		while ( iter.hasNext() ) {
			buf.append( ( (Column) iter.next() ).getQuotedName(dialect) );
			if ( iter.hasNext() ) buf.append(StringHelper.COMMA_SPACE);
		}
		buf.append(StringHelper.CLOSE_PAREN);
		String constraintString = StringHelper.replace( buf.toString(), "primary key", "unique" ); //TODO: improve this hack!
		/*
		 * Modification by Network Appliance to ensure all constraints are initially deferred.
		 */
		if (dialect instanceof Oracle9Dialect) {
			constraintString += " initially deferred deferrable ";
		}
		return constraintString;
	}
    
    public String sqlCreateString(Dialect dialect, Mapping p, String defaultSchema) {
        if ( dialect.supportsUniqueConstraintInCreateAlterTable() ) {
            return super.sqlCreateString(dialect, p, defaultSchema);
        } 
        else {
            return Index.buildSqlCreateIndexString(dialect, getName(), getTable(), getColumnIterator(), true, defaultSchema);
        }
    }
    
    public String sqlDropString(Dialect dialect, String defaultSchema) {
        if( dialect.supportsUniqueConstraintInCreateAlterTable() ) {
            return super.sqlDropString(dialect, defaultSchema);
        } 
        else {
            return Index.buildSqlDropIndexString(dialect, getTable(), getName(), defaultSchema);
        }
    }
}







